'<TiCoBasic Header, Headerversion 001.001>
'<Header End>
'ADBASIC-SYSTEM-FILE--V1.00--0000--0000--INC--PRO
'
'##############################################################################
'#                        AINTico.inc                                         #
'#              INCLUDE-file for AIN with Tico                                #
'##############################################################################
'#                                                                            #
'#                                                                            #
'#  04.09.2006  mf   creation                                                 #
'#  01.02.2008  mf   #if adwin_system                                         #
'#  25.03.2008  mf   new adr                                                  #
'#  25.02.2009  mf   '''+                                                     #
'#  18.03.2010  mf   wait_mux                                                 #
'#                                                                            #
'# Attention:                                                                 #
'#   - Do not copy the first source line (containing                          #
'#     "'ADBASIC-SYSTEM-FILE...") to user files !                             #
'#     Otherwise the Syntax color may be not correct.                         #
'#   - This includefile requires ADbasic 5 or higher!                         #
'#  Eventsource with clear (108041E0H) without clear (108041E1H) Bit 16       #
'##############################################################################
 
#if adwin_system=ADwin_ProII
#else
#error("This include file can only be used with the ADwin ProII")  
#endif


'******************************************************************************
''' Reads the event inputs (channel 0 - 31)
'''****************************************************************************
FUNCTION Event_Read() as Long 
  Event_Read=in(7040h)
ENDFUNCTION



'*******************************************************************************
'''  activates or deactivates the external Event input
'''  value: 
'''+       0 deactivate
'''+       1 activate
'''*****************************************************************************
SUB Event_Enable(value)
  DIM old AS Long

  old=in(80804800h)
  old =(old AND 0FFFFFFFEH) or value
  out (80804800h, old)
ENDSUB

'*******************************************************************************
'''  config the external Event input
'''  min_hold 
'''+          0=15 ns (default) 
'''+          1 =50 ns
'''  edge: 
'''+          1 rising edge (default)
'''+          2 falling edge
'''+          3 rising & falling edge
'''  prescale: 1 to 255
'''*****************************************************************************
SUB Event_Config(min_hold,edge,prescale)
  DIM old AS Long
  DIM neu AS Long
  DIM value AS Long

  old=in(80804800h)

  neu=shift_left((min_hold AND 1),4) or shift_left((edge AND 3),2)
  old =(old AND 0FFFFFFE3H) or neu
  out (80804800h, old)
  value=prescale AND 0FFH
  out (80804810h,value)
ENDSUB

'*******************************************************************************
''' creat an event for the master cpu
'''*****************************************************************************
SUB Trigger_Event()
  out (108041E0h, 1)
ENDSUB

'******************************************************************************
''' switches a LED on or off
''' value: 
'''+      0 off
'''+      1 on
'''****************************************************************************
SUB SET_LED(value) 
  out (80804000H, value)
ENDSUB

'******************************************************************************
'''  checks the LED 
'''****************************************************************************
FUNCTION CHECK_LED() AS LONG
  CHECK_LED=in(80804000h)		
ENDFUNCTION


'******************************************************************************
''' Set MUX to desired input channel.
''' select: 
'''+    bit 4...0 -> channel 1...32
'''+    bit 9 & 8 -> gain  
'''****************************************************************************
SUB SET_MUX(select)
  OUT(5010h,select)  
ENDSUB

'********************************************************************************
'''wait the difference time between the last change of multiplexer and required time of the multiplexer
'''+the required time can be set with p2_seq_init
'''******************************************************************************
SUB Wait_Mux()
  DIM muxtime AS Long
  '............ wait multiplexer settling time ........
  nops(4)
  
  '............ wait multiplexer settling time ........
  muxtime=IN(5022h) '
  
  if (muxtime>6) then 
    SLEEP((muxtime/5)) 															
  endif

ENDSUB




'******************************************************************************
''' Selects single-ended (s.e.) or differential (diff.) inputs
'''+ for the AIN-32/x-module only.
''' mode: 
'''+      0     s.e.            
'''+      1     diff. (default after power-up)
'''****************************************************************************
SUB SE_DIFF(mode)
  OUT(5012h,mode)
ENDSUB
  

'******************************************************************************
''' Start conversion of ADC.
'''****************************************************************************
SUB START_CONV()
  OUT(00007000h,1)
ENDSUB
  
'******************************************************************************
''' Wait until EOC-signal goes active (End Of Conversion).
'''****************************************************************************
SUB WAIT_EOC()
  DIM reminder AS LONG
     
  '............ wait EOC ............
  nops(3)
  
  do
    reminder= IN(00007010h)	
  until ((reminder AND 1)=0) 
ENDSUB


'******************************************************************************
''' Returns conversion result of ADC 
'''+result: 16 bit integer
'''****************************************************************************
FUNCTION READ_ADC() AS LONG
  READ_ADC=IN(00000200h)	
ENDFUNCTION

'******************************************************************************
''' Returns conversion result of ADV
'''+result: 24 bit integer
'''****************************************************************************
FUNCTION READ_ADC24() AS LONG
  READ_ADC24=IN(00000000h)	 
ENDFUNCTION

'******************************************************************************
''' Samples, converts and reads out the digital ADC-value of an analog input.
'''+ Attention:                                                                
'''+ This instruction is not prepared to handle an access from                 
'''+ a low AND high priority process to the same module.
'''+ result: 16 bit integer         
''' ADC_No: channel number of analog input
'''****************************************************************************
FUNCTION ADC(ADC_No) AS LONG
 
  DIM reminder AS LONG
  DIM muxtime AS LONG
          
  '............ set MUX ............ 
  OUT( 5010h,(ADC_No-1) )
 
  nops(4)
  
  '............ wait multiplexer settling time ........
  muxtime=IN(5022h) '
  
  if (muxtime>6) then 
    SLEEP((muxtime/5)) 															
  endif

  '............ start conversion ............
        
  OUT(00007000h,1)
		
  ' folgende nops sind erforderlich, aufgrund fpga-interner zwischentaktung
  ' zur Verbesserung des fpga-timings.
  ' => unkritisch, da sowieso gewartet wird !
		
  nops(3) ' Out-Datum steht am Beginn diese NOP bereits an  __|--
  ' Busy_adc (=STCONV) auf '1' zu Beginn dieses NOP ____|--
  ' Busy_out           auf '1' zu Beginn dieses NOP ______|--
  ' => Busy_out kann mit dem nchsten IN-Befehl gelesen werden
				
  '............ wait EOC ............
  do
    ' EOC, d.h. Busy_out='0' mit 1-Taktzyklus langem In-Befehl erkennen:
    reminder= IN(00007010h)	
  until ((reminder AND 1)=0) 
 
  '............ read out result ............
 
  ADC=IN(00000200h)	
ENDFUNCTION


'******************************************************************************
''' Samples, converts and reads out the digital ADC-value of an analog input.
'''+ Attention:                                                           
'''+ This instruction is not prepared to handle an access from             
'''+ a low AND high priority process to the same module.                      
''' ADC_No: channel number of analog input
''' result: 24 bit integer
'''****************************************************************************
FUNCTION ADC24(ADC_No) AS LONG
  DIM reminder AS LONG
  DIM muxtime AS LONG
          
  '............ set MUX ............
  OUT(5010h,ADC_No-1)
  '............ wait multiplexer settling time ........
  nops(4)
  muxtime=IN(5022h)
	

  if (muxtime>6) then 
    SLEEP((muxtime/5)) 															
  endif

       
  '............ start conversion ............
        
  OUT(00007000h,1)
		
  ' folgende nops sind erforderlich, aufgrund fpga-interner zwischentaktung
  ' zur Verbesserung des fpga-timings.
  ' => unkritisch, da sowieso gewartet wird !
		
  'nop ' Out-Datum steht am Beginn diese NOP bereits an  __|--
  'nop	' Busy_adc (=STCONV) auf '1' zu Beginn dieses NOP ____|--
  'nop	' Busy_out           auf '1' zu Beginn dieses NOP ______|--
  ' => Busy_out kann mit dem nchsten IN-Befehl gelesen werden
  nops(3)

		
  '............ wait EOC ............
  do		
    reminder= IN(00007010h) 

  until ((reminder AND 1)=0) 
	
  '............ read out result ............
		 
  ADC24=IN(00000000h)	
		
ENDFUNCTION


'******************************************************************************
''' Returns conversion result of ADC and starts immediately new conversion.
'''+result: 16 bit long
'''****************************************************************************
FUNCTION READ_ADC_SCONV() AS LONG
  READ_ADC_SCONV=IN(00000300h)	
ENDFUNCTION

'******************************************************************************
''' Returns conversion result of ADC and starts immediately a new conversion.
'''+result: 24 bit long
'''****************************************************************************
FUNCTION READ_ADC_SCONV24() AS LONG
  READ_ADC_SCONV24=IN(00000100h)	
ENDFUNCTION
