/*********************************************************************************************
	getDataSH.c
	MB 01.02.2005
	GetData �ber Utopia mit versetztem Handshake (shifted handshake SH)
	Windows: Ws2_32.lib wird ben�tigt (Funktionen htonl, htos, ntohl, ntohs)
		
*********************************************************************************************/


#include <math.h>
#include <float.h>
#include <string.h>
#include <stdio.h>
#include <fcntl.h>
#include <sys/types.h>
#include <stdlib.h>
#include <stddef.h>
#include <sys/timeb.h>
#include <time.h>

#if defined (WIN32) || defined (__BORLANDC__)

#include <windows.h>
#include <io.h>
#include <winioctl.h>
#include <winreg.h>

#include "FehlerText.h"
#include "UDPADwin.h"

#else

#include <netinet/in.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/select.h>
#include <errno.h>
#include "UDPADwin.h"

#endif
#include "data_convert.h"

#include "getDataSH.h"


#if VERBOSE_LEVEL > 0
#ifdef linux
#define fp stdout
#else
FILE *fp;
#endif
int errCount;
#endif


int queueGetMin(s_getDataQueue * getDataQueue)
{
	int i, min;

	min = 0;

	// min auf ersten Wert != 0 setzen
	for (i = 0; i < QUEUESIZE; i++)
	{
		if (getDataQueue->queue[i])
		{
			min = getDataQueue->queue[i];
			break;
		}
	}
	if (min == 0)
		return 0;				// queue ist leer

	for (; i < QUEUESIZE; i++)
		if (getDataQueue->queue[i] && (getDataQueue->queue[i] < min))
			min = getDataQueue->queue[i];

	return min;
}


int queueGetMax(s_getDataQueue * getDataQueue)
{
	int i, max;

	max = 0;
	for (i = 0; i < QUEUESIZE; i++)
		if (getDataQueue->queue[i] > max)
			max = getDataQueue->queue[i];

	return max;
}


void queuePrint(s_getDataQueue * getDataQueue)
{
#if VERBOSE_LEVEL > 0
	int i;

	fprintf(fp, "PC-Liste:\n");
	for (i = 0; i < QUEUESIZE; i++)
		fprintf(fp, "%5d: %d\n", i, getDataQueue->queue[i]);
#else
	return;
#endif
}


void queueDeleteAll(s_getDataQueue * getDataQueue)
{
	int i;

	for (i = 0; i < QUEUESIZE; i++)
		getDataQueue->queue[i] = 0;
}


int queueDeleteItem(s_getDataQueue * getDataQueue, int item)
{
	int i;

	// erstmal an der Position des Zeigers nachschauen, ob sich dort der Eintrag befindet
	if (getDataQueue->queue[getDataQueue->queueDeletePointer] == item)
	{
		getDataQueue->queue[getDataQueue->queueDeletePointer] = 0;
		getDataQueue->queueDeletePointer++;
		if (getDataQueue->queueDeletePointer >= QUEUESIZE)
			getDataQueue->queueDeletePointer = 0;
		return 0;
	}
	// schauen, wo sich der Eintrag in der Liste befindet
	for (i = 0; i < QUEUESIZE; i++)
	{
		if (getDataQueue->queue[i] == item)
		{
			getDataQueue->queue[i] = 0;
			getDataQueue->queueDeletePointer = i + 1;
			if (getDataQueue->queueDeletePointer >= QUEUESIZE)
				getDataQueue->queueDeletePointer = 0;
			return 0;
		}
	}
	// Eintrag befindet sich nicht in der Liste
#if VERBOSE_LEVEL > 1
	fprintf(fp, "%d doppelt!\n", item);
	queuePrint(getDataQueue);
#endif
	return 1;
}


int queueInsertItem(s_getDataQueue * getDataQueue, int item)
{
	int full = 0;

#if VERBOSE_LEVEL > 1
	fprintf(fp, "queueInsertItem(%d)\n", item);
#endif

	while (1)
	{
		if (getDataQueue->queue[getDataQueue->queueInsertPointer] == 0)
		{
			// leere Stelle, alles ok
			getDataQueue->queue[getDataQueue->queueInsertPointer] = item;
			getDataQueue->queueInsertPointer++;
			if (getDataQueue->queueInsertPointer >= QUEUESIZE)
				getDataQueue->queueInsertPointer = 0;
			return 0;
		}
		else
		{
			// hier befindet sich die Nummer eines Paketes, das noch nicht angekommen ist
#if VERBOSE_LEVEL > 0
			fprintf(fp, "*** %d\n", getDataQueue->queue[getDataQueue->queueInsertPointer]);
#endif
			// wurde ev. durch queueSendAllLessThan nochmal angefordert
			// TODO: wenn nicht: nochmal anfordern
			//sendRequest(getDataQueue, 0, getDataQueue->queue[getDataQueue->queueInsertPointer], 0, 0);

			getDataQueue->queueInsertPointer++;

			if (getDataQueue->queueInsertPointer >= QUEUESIZE)
				getDataQueue->queueInsertPointer = 0;

			full++;
			if (full == QUEUESIZE)
			{
#if VERBOSE_LEVEL > 1
				fprintf(fp, "*** queue voll!\n");
				queuePrint(getDataQueue);
#endif
				return -1;		// Liste komplett voll
			}
		}
	}
	return 0;
}


int queueSendAll(s_getDataQueue * getDataQueue)
{
	int i;

	for (i = 0; i < QUEUESIZE; i++)
	{
		if (getDataQueue->queue[i] != 0)
		{

#if VERBOSE_LEVEL > 1
			fprintf(fp, "queueSendAll: %d\n", getDataQueue->queue[i]);

#endif

			if (sendRequest(getDataQueue, 0, getDataQueue->queue[i], 0, 0))

			{
				// Systemfehler
				getDataQueue->errorCode = k_error_tcp_error;
				return -1;
			}
		}
	}
	return 0;
}


int queueSendAllLessThan(s_getDataQueue * getDataQueue, int item)
{
	int i;

	for (i = 0; i < QUEUESIZE; i++)
	{
		if (getDataQueue->queue[i] && (getDataQueue->queue[i] < item))
		{

#if VERBOSE_LEVEL > 1
			fprintf(fp, "queueSendAllLessThan(%d): %d\n", item, getDataQueue->queue[i]);

#endif

			if (sendRequest(getDataQueue, 0, getDataQueue->queue[i], 0, 0))
			{
				// Systemfehler
				getDataQueue->errorCode = k_error_tcp_error;
				return -1;
			}
		}
	}
	return 0;
}


int sendRequest(s_getDataQueue * getDataQueue, uint32_t isRange, uint32_t p1, uint32_t p2, int enlist)
{
	GetDataSHRequest messageOut;
	int n, min = 0;
	uint32_t i;

	memset(&messageOut.dummy, 0, 32);
	getDataQueue->requestNumber++;
	messageOut.packetIndex = htonl(getDataQueue->packetIndex);
	messageOut.requestNumber = htonl(getDataQueue->requestNumber);


	if (isRange != 2)
	{
		min = queueGetMin(getDataQueue);

		if (min == 0)
		{
			min = getDataQueue->lastReceived;

#if VERBOSE_LEVEL > 1
			fprintf(fp, "*** sendRequest: queue ist leer, setze completePackages auf %d\n", min);

#endif
		}
		else
			min = min - 1;
	}

#if VERBOSE_LEVEL > 1
	fprintf(fp, "sendRequest: requestNumber: %lx, complete: %d, ", getDataQueue->requestNumber, min);

	switch (isRange)
	{
		case 0:
			fprintf(fp, "verlange %ld\n", p1);
			break;
		case 1:
			fprintf(fp, "verlange von %ld bis %ld\n", p1, p2);
			break;
		case 2:
			fprintf(fp, "schicke Ende\n");
	}
#endif

	messageOut.completePackages = htonl(min);
	messageOut.isRange = htonl(isRange);
	messageOut.p1 = htonl(p1);
	messageOut.p2 = htonl(p2);

	if (enlist)
	{
		if (isRange)
			for (i = p1; i <= p2; i++)
				queueInsertItem(getDataQueue, i);
		else
			queueInsertItem(getDataQueue, p1);
	}

#ifdef GDSH_DEBUG
	{
		if (rand() < GDSH_DEBUG_LEVEL)
		{
#if VERBOSE_LEVEL > 1
			if (isRange)
				fprintf(fp, "\t\t\t\tausgelassen: isRange: %ld, %ld - %ld\n", isRange, p1, p2);
			else
				fprintf(fp, "\t\t\t\tausgelassen: %ld\n", p1);
#endif
		}
		else
		{
			n = sendto(getDataQueue->sockfd, (char *) &messageOut, sizeof(messageOut), 0, getDataQueue->serverAddr, sizeof(struct sockaddr));
			if ((n == -1) || (n != sizeof(messageOut)))
			{

#if VERBOSE_LEVEL > 0
				fprintf(fp, "*** sendto: %s\n", strerror(errno));

#endif
				return -1;
			}
		}
	}
#else
	{
		n = sendto(getDataQueue->sockfd, (char *) &messageOut, sizeof(messageOut), 0, getDataQueue->serverAddr, sizeof(struct sockaddr));
		if ((n == -1) || (n != sizeof(messageOut)))
		{

#if VERBOSE_LEVEL > 0
			fprintf(fp, "*** sendto: %s\n", strerror(errno));
#endif
			return -1;
		}
	}
#endif
	return 0;
}


int getarray_shifted_handshake(s_getDataQueue *getDataQueue, int32_t timeout, int totalCount, int32_t *data, uint16_t minReq, uint16_t maxReq, int *retry_versuche) // totalCount = Anzahl in Longs
{
	uint32_t isRange, p1, p2;

	int openPackets = 0;

	uint32_t nextPacket;	// erwartete Paketnummer
	int lastPacket;				// letztes angefordertes Paket
	int packetCount;			// Anzahl Pakete gesamt
	int lastPacketSize;			// Anzahl von benutzten Longs im letzten Paket
	fd_set fdSet;
	int retSelect;
	int recvCount;
	struct timeval wait = { 0, 0 };
	char recvBuffer[2000];
	uint32_t receivedPacket;	// angekommenes Paket;
	socklen_t server_len;
	uint32_t i;

#if VERBOSE_LEVEL > 0
	errCount = 0;
#endif

#ifdef linux
#if VERBOSE_LEVEL > 0
	fp = stdout;
#endif
#else
#if VERBOSE_LEVEL == 1
	fp = fopen("c:\\adwin\\dll_check.log", "a");	// append
#elif VERBOSE_LEVEL == 2
	fp = fopen("c:\\adwin\\dll_check.log", "w");
#endif
#endif

	server_len = sizeof(struct sockaddr);
	getDataQueue->lastReceived = 0;

	packetCount = totalCount / PACKAGESIZE;
	if (totalCount % PACKAGESIZE)
		packetCount++;
	lastPacketSize = totalCount % PACKAGESIZE;
	if (lastPacketSize == 0)
		lastPacketSize = PACKAGESIZE;

	queueDeleteAll(getDataQueue);

#if VERBOSE_LEVEL > 1
	fprintf(fp, "totalCount: %d, packetCount: %d, lastPacketSize: %d\n", totalCount, packetCount, lastPacketSize);
#endif

	isRange = 1;				// Reihe
	p1 = 1;
	p2 = (packetCount <= maxReq) ? packetCount : maxReq;

	for (i = 1; i <= p2; i++)
		queueInsertItem(getDataQueue, i);

	openPackets = p2;
#if VERBOSE_LEVEL > 1
	fprintf(fp, "openPackets: %d\n", openPackets);
#endif
	nextPacket = 1;
	lastPacket = maxReq > packetCount ? packetCount : maxReq;

	getDataQueue->timeoutCounter = 0;

	while (openPackets)
	{
		// Port abfragen
		FD_ZERO(&fdSet);
		FD_SET(getDataQueue->sockfd, &fdSet);

		wait.tv_sec = timeout / 1000;
		wait.tv_usec = (timeout - 1000 * wait.tv_sec) * 1000;
		retSelect = select(getDataQueue->sockfd + 1, &fdSet, (fd_set *) 0, (fd_set *) 0, &wait);
		
		if (retSelect == -1)
		{

#if VERBOSE_LEVEL > 0
			fprintf(fp, "*** Fehler beim Select: %d\n", retSelect);
			fprintf(fp, "*** select: %s\n", strerror(errno));

			errCount++;
#endif

#if VERBOSE_LEVEL > 0
			if (errCount)
				fprintf(fp, "-\n");
#ifndef linux
			fclose(fp);
#endif
#endif
			getDataQueue->errorCode = k_error_tcp_error;

			return -1;
		}

		if (retSelect == 0)		// timeout
		{
			getDataQueue->timeoutCounter++;
#if VERBOSE_LEVEL > 0
			fprintf(fp, "*** timeout %d, openPackets: %d\n", getDataQueue->timeoutCounter, openPackets);
			errCount++;
#endif
			if (getDataQueue->timeoutCounter >= MAXTIMEOUTS)
			{
#if VERBOSE_LEVEL > 0
				fprintf(fp, "*** Ende wegen zu vieler Timeouts!\n");
				if (errCount)
					fprintf(fp, "-\n");
#ifndef linux
				fclose(fp);

#endif
#endif
				getDataQueue->errorCode = k_error_tcp_timout_read;
				//return k_UDPtimeout;
				return -1;
			}
			*retry_versuche = *retry_versuche + 1;

			// alle offenen Pakete nochmal anfordern
			if (queueSendAll(getDataQueue))
				return -1;
		}

		if (retSelect > 0)
		{
			// etwas angekommen
			recvCount = recvfrom(getDataQueue->sockfd, recvBuffer, sizeof(recvBuffer), 0, getDataQueue->serverAddr, &server_len);
#if VERBOSE_LEVEL > 1
			if (recvCount != sizeof(GetDataSHPacket))
				fprintf(fp, "recvCount: %d Bytes\n", recvCount);
#endif

			// analysieren
			// stimmt Gr��e?
			if (recvCount == sizeof(GetDataSHPacket))
			{
				// stimmt Paketindex?
				if (ntohl(((GetDataSHPacket *) recvBuffer)->packetIndex) == getDataQueue->packetIndex)
				{
					getDataQueue->timeoutCounter = 0;

					// stimmt Paketnummer?
					receivedPacket = ntohl(((GetDataSHPacket *) recvBuffer)->packetNumber);
					getDataQueue->lastReceived = receivedPacket;
#if VERBOSE_LEVEL > 0				
					if (receivedPacket != nextPacket)
					{
						fprintf(fp, "*** %ld statt %ld\n", receivedPacket, nextPacket);
						errCount++;
					}
#endif				
#if VERBOSE_LEVEL > 1
					fprintf(fp, "-> %ld\n", receivedPacket);
#endif
					// Daten �bernehmen...
					if (!queueDeleteItem(getDataQueue, receivedPacket))
					{
						// Paket war in der Queue
						openPackets--;
						// Daten kopieren
						// letztes Paket: weniger als 1400 Byte...
						if (receivedPacket == (uint32_t) packetCount)
							memcpy_letohl(&data[(receivedPacket - 1) * PACKAGESIZE], ((GetDataSHPacket *) & recvBuffer)->data, lastPacketSize * 4);
						else
							memcpy_letohl(&data[(receivedPacket - 1) * PACKAGESIZE], ((GetDataSHPacket *) & recvBuffer)->data, PACKAGESIZE * 4);
					}
					nextPacket = receivedPacket + 1;
#if VERBOSE_LEVEL > 1
					fprintf(fp, "openPackets: %d\n", openPackets);
#endif
				} // packetIndex
#if VERBOSE_LEVEL > 1
				else
					fprintf(fp, "*** Paketindex stimmt nicht; Soll: %ld, Ist: %d\n", getDataQueue->packetIndex, ntohl(((GetDataSHPacket *) recvBuffer)->packetIndex));
#endif
			} // recvCount
			
#if VERBOSE_LEVEL > 1
			else
				fprintf(fp, "recvCount: %d (Soll: %d)\n", recvCount, sizeof(GetDataSHPacket));
#endif
		} // retSelect > 0

		// neue Anforderung verschicken?
		if ((openPackets == minReq) && (lastPacket != packetCount))
		{
			p1 = lastPacket + 1;
			if ((packetCount - lastPacket) >= (maxReq - minReq))	// noch nicht angeforderte Pakete
			{
				// maxReq-minReq Pakete anfordern
				p2 = lastPacket + (maxReq - minReq);
				openPackets += (maxReq - minReq);
			}
			else if (lastPacket < packetCount)
			{
				// Rest anfordern

#if VERBOSE_LEVEL > 1
				fprintf(fp, "Rest anfordern...\n");
#endif
				p2 = packetCount;
				openPackets += (packetCount - lastPacket);
			}

			// fehlen noch Paketnummern kleiner als die letzte Anforderung?
			if (queueSendAllLessThan(getDataQueue, lastPacket - minReq))
				return -1;

			lastPacket = p2;
			isRange = (p1 == p2) ? 0 : 1;

			if (sendRequest(getDataQueue, isRange, p1, p2, 1))
			{
				// Systemfehler
				getDataQueue->errorCode = k_error_tcp_error;
				return -1;
			}

#if VERBOSE_LEVEL > 1
			fprintf(fp, "openPackets: %d\n", openPackets);

#endif
		}
	}							// while openPackets

	// Ende an Server schicken
	if (sendRequest(getDataQueue, 2, 0, 0, 0))
	{

#if VERBOSE_LEVEL > 0
		if (errCount)
			fprintf(fp, "-\n");

#ifndef linux
		fclose(fp);

#endif
#endif
		//getDataQueue->errorCode = k_error_tcp_error;
		//return -1;
		return 0;				// f�r client alles ok, nur das Senden des letzten ACKs hat nicht geklappt
	}

#if VERBOSE_LEVEL > 0
	if (errCount)
		fprintf(fp, "-\n");

#ifndef linux
	fclose(fp);
#endif
#endif

	return 0;
}
