#ifndef __LIBADWIN_MACROS_H__
#define __LIBADWIN_MACROS_H__

#include <string.h>

#ifdef BUILD_LIBRARY

/*
 Die Case - Makros sind dazu gedacht, in eine switch (dev.dev_type |
 dev.dev_processor) - Anweisung eingebaut zu werden. Sie bilden einen
 Funktionsnamen wie "DSP_Get_Par" und so weiter.
*/

#if defined(WITH_ETHERNET)
	#define CASE_NET(retval, func, args...)		\
		case DP_DSP | DC_ETHERNET:		\
		case DP_UNKNOWN | DC_ETHERNET:		\
			retval = NET_##func(args);	\
			break;
#else
	#define CASE_NET(retval, func, args...)
#endif

#if defined(WITH_LINK)
	#define CASE_DSP(retval, func, args...)		\
		case DP_DSP | DC_LINK:			\
			retval = DSP_##func(args);	\
			break;

	#define CASE_TRANSPUTER(retval, func, args...)	\
		case DP_T2 | DC_LINK:			\
		case DP_T4 | DC_LINK:			\
		case DP_T8 | DC_LINK:			\
			retval = T_##func(args);	\
			break;
#else
	#define CASE_DSP(retval, func, args...)
	#define CASE_TRANSPUTER(retval, func, args...)
#endif

#if defined(WITH_USB)
	#define CASE_USB(retval, func, args...)		\
		case DP_DSP | DC_USB:			\
		case DP_UNKNOWN | DC_USB:		\
			retval = USB_##func(args);	\
			break;
#else
	#define CASE_USB(retval, func, args...)
#endif

#define SELECT_PROTOCOL(retval, func, dev, args...) 	\
	switch((dev)->dev_processor | (dev)->dev_type){	\
		CASE_NET(retval, func, dev, args)	\
		CASE_DSP(retval, func, dev, args)	\
		CASE_USB(retval, func, dev, args)	\
		CASE_TRANSPUTER(retval, func, dev, args)\
		default:				\
			errno = EINVAL;			\
			adwin_errno = -EINVAL;		\
			retval = -1;			\
	}


#define SELECT_PROTOCOL_DATA(GetSet, retval, dev, fd, data_no, start_index, count, void_ptr, type)     \
	switch((dev)->dev_processor | (dev)->dev_type){	\
		CASE_NET(retval, GetSet##Data, dev, fd, data_no, start_index, count, void_ptr, type)	\
		CASE_DSP(retval, GetSet##Data, dev, fd, data_no, start_index, count, void_ptr)	\
		CASE_USB(retval, GetSet##Data, dev, fd, data_no, start_index, count, void_ptr)	\
		CASE_TRANSPUTER(retval, GetSet##Data, dev, fd, data_no, start_index, count, void_ptr) \
		default:				\
			errno = EINVAL;			\
			adwin_errno = -EINVAL;		\
			retval = -1;			\
	}

#define SELECT_PROTOCOL_FIFO(GetSet, retval, dev, fd, data_no, count, void_ptr, type)     \
	switch((dev)->dev_processor | (dev)->dev_type){	\
		CASE_NET(retval, GetSet##Fifo, dev, fd, data_no, count, void_ptr, type)	\
		CASE_DSP(retval, GetSet##Fifo, dev, fd, data_no, count, void_ptr)	\
		CASE_USB(retval, GetSet##Fifo, dev, fd, data_no, count, void_ptr)	\
		CASE_TRANSPUTER(retval, GetSet##Fifo, dev, fd, data_no, count, void_ptr) \
		default:				\
			errno = EINVAL;			\
			adwin_errno = -EINVAL;		\
			retval = -1;			\
	}

#endif

/** 
 * CAST casts "cnt" elements of type "from_type" at position
 * "from_ptr" to type "to_type" at position "to_ptr". Wear a helmet.
 * TB
 * 
 * @param from_type type of data to cast.
 * @param from_ptr pointer to start reading from.
 * @param to_type desired type of cast.
 * @param to_ptr pointer to start writing to.
 * @param cnt number of elements to cast.
 */
#define CAST(from_type, from_ptr, to_type, to_ptr, cnt) 		\
do {									\
	unsigned long i;								\
	from_type *__fptr = (from_type *) (from_ptr);			\
	to_type   *__tptr = (to_type *) (to_ptr);			\
	for (i = 0; i < (cnt); i++)					\
		*(__tptr)++ = (to_type) *__fptr++;			\
} while(0)

#define CAST_FROM_VARIANT(from_ptr, to_type, to_ptr, cnt)\
do {\
	unsigned long i;\
	struct myVariant *__fptr = from_ptr;\
	to_type *__tptr = (to_type *) (to_ptr);\
	for (i = 0; i < (cnt); i++) {\
		switch (__fptr->type) {\
		case VARIANT_SHORT: {\
			int16_t *tmp = (void*)(__fptr->value);\
			*__tptr = (to_type)*tmp;\
			break;\
		}\
		case VARIANT_LONG: {\
			int32_t *tmp = (void*)(__fptr->value);\
			*__tptr = (to_type)*tmp;\
			break;\
		}\
		case VARIANT_FLOAT: {\
			float *tmp = (void*)(__fptr->value);\
			*__tptr = (to_type)*tmp;\
			break;\
		}\
		case VARIANT_DOUBLE: {\
			double *tmp = (void*)(__fptr->value);\
			*__tptr = (to_type)*tmp;\
			break;\
		}\
		default:\
			*__tptr = 0;\
		}\
		__tptr++;\
		__fptr++;\
	}\
} while(0)

#define CAST_TO_VARIANT(from_type, from_ptr, to_var_type, to_ptr, cnt)\
do {\
	unsigned long i;\
	from_type *__fptr = (from_type *) (from_ptr);\
	struct myVariant *__tptr = to_ptr;\
	for (i = 0; i < (cnt); i++) {\
                __tptr->type = to_var_type;\
		switch (to_var_type) {\
		case VARIANT_SHORT: {\
			int16_t *tmp = (int16_t *)((void*)(__tptr->value));\
			*tmp = *__fptr;\
			break;\
		}\
		case VARIANT_LONG: {\
			int32_t *tmp = (int32_t *)((void*)(__tptr->value));\
			*tmp = *__fptr;\
			break;\
		}\
		case VARIANT_FLOAT: {\
			float *tmp = (float *)((void*)(__tptr->value));\
			*tmp = *__fptr;\
			break;\
		}\
		case VARIANT_DOUBLE: {\
			double *tmp = (double *)((void*)(__tptr->value));\
			*tmp = *__fptr;\
			break;\
		}\
		default:\
                        memset(__tptr->value, 0, sizeof(__tptr->value)); /* should not happen! */ \
		}\
		__tptr++;\
		__fptr++;\
	}\
} while(0)


/** 
 * FCAST is similar to CAST, but designed to cast from float values to
 * other values. It maps all NANs to MAXINT or something similar.
 */
#define g_QNAN_MASK 0x7F800000
#define FCAST(from_type, from_ptr, to_type, to_ptr, cnt) 		\
do {									\
	unsigned long i;								\
	from_type *__fptr = (from_type *) (from_ptr);			\
	to_type   *__tptr = (to_type *) (to_ptr);			\
	for (i = 0; i < (cnt); i++, __tptr++, __fptr++)			\
		if ((*((unsigned int *)__fptr) & g_QNAN_MASK) != g_QNAN_MASK)	\
			*(__tptr) = (to_type) *(__fptr);		\
		else							\
			*(__tptr) = (to_type)(0x7FFFFFFFFFFFFFFFLL >> (64 - sizeof (to_type) * 8));	\
} while (0)

#endif /* __LIBADWIN_MACROS_H__ */
