#ifndef __ADCTL_H__
#define __ADCTL_H__

#include <sys/types.h>
#include <sys/ioctl.h>

#include <netinet/in.h>

#ifdef __cplusplus
extern "C" {
#endif

extern int adctl(int request, ...);

#ifdef __cplusplus
}
#endif

struct adconf {
	size_t adc_len;					/* size of buffer		*/
	union {
		caddr_t adcu_buf;
		struct adreq *adcu_req;
	} adc_adcu;
};
#define adc_buf		adc_adcu.adcu_buf		/* buffer address (in)		*/
#define adc_req		adc_adcu.adcu_req		/* array of structures (out)	*/


#define ADWIN_DEVNAMESIZE	 8
#define ADWIN_DEVDESCSIZE	32
#define ADWIN_DEVPWSIZE		12

struct adreq {
	char adr_name[ADWIN_DEVNAMESIZE];		/* device name, e.g. "0x150"	*/

	union {
		char adru_desc[ADWIN_DEVDESCSIZE];
		uid_t adru_uid;
		gid_t adru_gid;
		mode_t adru_mode;

		int adru_type;
		int adru_link_base_addr;
		int adru_usb_serial;
		struct sockaddr_in adru_server;
		
		int adru_timeout;
		int adru_count;
		int adru_hostlink;
		char adru_pw[ADWIN_DEVPWSIZE];

		caddr_t adru_buf;
	} adr_adru;
};
#define adr_buf		adr_adru.adru_buf		/* buffer address (in)		*/
#define adr_desc	adr_adru.adru_desc		/* device specific text.	*/
#define adr_uid		adr_adru.adru_uid		/* owner of device		*/
#define adr_gid		adr_adru.adru_gid		/* group of device		*/
#define adr_mode	adr_adru.adru_mode		/* mode of device		*/

#define adr_type	adr_adru.adru_type		/* type of dev {link, usb, net} */
#define adr_base	adr_adru.adru_base		/* base addr of link dev	*/
#define adr_serial	adr_adru.adru_serial		/* serial # of usb dev		*/
#define adr_server	adr_adru.adru_server		/* ip/port of net dev		*/

#define adr_timeout	adr_adru.adru_timeout		/* UDP timeout			*/
#define adr_count	adr_adru.adru_count		/* UDP package count		*/
#define adr_hostlink	adr_adru.adru_hostlink		/* server link			*/
#define adr_pw		adr_adru.adru_pw		/* network system password	*/


/*
 * adctl() request constants
 */
#define ADIOGDEVNUM	_IOW(0xAD,  0, int)		/* number of devices		*/
#define ADIOGDEVCONF	_IOW(0xAD,  1, struct adconf)	/* get list of configured dev's	*/

#define ADIOGDEVDESC	_IOW(0xAD,  2, struct adreq)	/* get/set device specific text	*/
#define ADIOSDEVDESC	_IOR(0xAD,  2, struct adreq)
#define ADIOGDEVUID	_IOW(0xAD,  3, struct adreq)	/* get/set owner of device	*/
#define ADIOSDEVUID	_IOR(0xAD,  3, struct adreq)
#define ADIOGDEVGID	_IOW(0xAD,  4, struct adreq)	/* get/set group of device	*/
#define ADIOSDEVGID	_IOR(0xAD,  4, struct adreq)
#define ADIOGDEVMODE	_IOW(0xAD,  5, struct adreq)	/* get/set mode of device	*/
#define ADIOSDEVMODE	_IOR(0xAD,  5, struct adreq)

#define ADIOGDEVTYPE	_IOW(0xAD,  6, struct adreq)	/* get/set type of device	*/
#define ADIOSDEVTYPE	_IOR(0xAD,  6, struct adreq)
#define ADIOGDEVBASE	_IOW(0xAD,  7, struct adreq)	/* get/set link base addr	*/
#define ADIOSDEVBASE	_IOR(0xAD,  7, struct adreq)
#define ADIOGDEVSERIAL	_IOW(0xAD,  8, struct adreq)	/* get/set usb serial		*/
#define ADIOSDEVSERIAL	_IOR(0xAD,  8, struct adreq)
#define ADIOGDEVSERVER	_IOW(0xAD,  9, struct adreq)	/* get/set server struct	*/
#define ADIOSDEVSERVER	_IOR(0xAD,  9, struct adreq)

#define ADIOGDEVTIMEO	_IOW(0xAD, 10, struct adreq)	/* get/set UDP timeout		*/
#define ADIOSDEVTIMEO	_IOR(0xAD, 10, struct adreq)
#define ADIOGDEVCOUNT	_IOW(0xAD, 11, struct adreq)	/* get/set UDP package count	*/
#define ADIOSDEVCOUNT	_IOR(0xAD, 11, struct adreq)
#define ADIOGDEVHSTLNK	_IOW(0xAD, 12, struct adreq)	/* get/set server link		*/
#define ADIOSDEVHSTLNK	_IOR(0xAD, 12, struct adreq)
#define ADIOGDEVPW	_IOW(0xAD, 13, struct adreq)	/* get/set server password	*/
#define ADIOSDEVPW	_IOR(0xAD, 13, struct adreq)



#endif /* __ADCTL_H__ */
