****************************************
*	 Prozenummer = 1
*	 Delay = 10000
*	 Eventsource = 0
*	 Number of Loops = 0
*	 Prioritt = 0
*	 Version = 1
*	 FastStop = 0
*	 AdbasicVersion = 4000001
*	 ATSRAM = 0
*	 OPT_LEVEL = 1
*	 SAVECOMPIL = 0
****************************************
' goldrpm2.bas
'
' H.M.   22.06.1999   creation
' JK     29.03.2000   max. input frequency is 300 kHz
' JK     19.09.2000   comments on accuracy , GLOBALDELAY
' R.Ra   16.03.2004   cnt_se_diff(3) added


' Demo-program for fast frequency / speed of  rotation measurement 
'   with ADwin-Gold-CO1
' The frequency is calculated on each successive measurement interval.
' Advantages of this program :
'  - high accuracies are achievable
'  - accuracy is constant on a wide range of the input frequency 

' In this mode the system can measure 2 Signals
' Counter#1 and #2 measure signal 1 
' Counter#3 and #4 measure signal 2
' Connect Signal 1 to cntr#1 Clock and cntr#2 latch
' Connect Signal 2 to cntr#3 clock and cntr#4 latch
' Connect CNTR#1 DIR and CNTR#3 DIR to ground
' The maximum input frequency is 300 kHz 
'     this is caused by the fact, that the period time of the input 
'     signal may NOT exceed the time between "time a" and "time b".
'     (The max. input frequency was experimentally determined to 
'     405 kHz. To have a safety margin for future changes in the 
'     ADbasic compiler, our statement to the customer is the limit
'     300 kHz.
'     (If input frequencies up to 1 MHz should be processed, please ask 
'     Jaeger Messtechnik for a optimized version of this program.)
' The accuracy of the ADwin-GOLD with this program can be calculated 
'  with the following formula :
'         average_accuracy = +- 50 ns / meas_interval * 100%
'         worst_case_accuracy = 2 * average_accuracy 
'
'     Example :       (measurement_interval = 500 microseconds)
'         average_accuracy = +- 0.01%
'         worst_case_accuracy = +- 0.02%
'     Note : This obviously does not include the accuracy of the
'            signal source itself (e.g. of the incremental encoder) 
' 
' Remark : If the period time of the input signal is bigger than 
'           the measurement_interval, the frequency is not updated
'           on each measurement_interval. 

' par_1  =  Signal 1  number of counts per measuring period
' par_2  =  Signal 1  precise time for this count (in 50 ns)
' fpar_1 =  Signal 1  frequency in kHz
' par_3  =  Signal 2  number of counts per measuring period
' par_4  =  Signal 2  precise time for this count (in 50 ns)
' fpar_2 =  Signal 2  frequency in kHz

#include adwgcnt.inc

dim ih, co1, co2, co3, co4 as integer
dim co1old, co2old, co3old, co4old as integer

init:
GLOBALDELAY = 10000  	' measurement_interval = 250 microseconds
cnt_enable(0)           ' disable all counters
cnt_clear(15)           ' clear all counters
cnt_se_diff(3)					' all counters with diff. inputs
cnt_mode(10)            ' counter 1 and 3 external, counter 2 and 4  internal clock
cnt_set(5)              ' counter 1 and 3 clock/dir, counter 2 and 4  20 MHz
cnt_inputmode(10)       ' counter 2 and 4 latch input
cnt_enable(15)          ' enable all counters


event:
                        ' "time a"
co1 = -cnt_read(1)      '  " - " because counter decrements with dir = 0
co2 = cnt_readlatch(2)
ih = - cnt_read(1)
if (ih <> co1) then        ' repeat if a count occurred while reading
  co1 = ih              
  co2 = cnt_readlatch(2)
                        ' "time b" 
  endif

par_1 = co1 - co1old      ' calculate number of counts
co1old = co1
par_2 = co2 - co2old      ' calculate time
co2old = co2
if ((par_1 <> 0) and (par_2 <> 0)) then fpar_1 = par_1/par_2 * 20000

                          ' do the same with counter 3 and 4
co3 = -cnt_read(3)
co4 = cnt_readlatch(4)
ih = - cnt_read(3)
if (ih <> co3) then
  co3 = ih
  co4 = cnt_readlatch(4)
  endif

par_3 = co3 - co3old
co3old = co3
par_4 = co4 - co4old
co4old = co4
if ((par_3<> 0) and (par_4 <> 0)) then fpar_2 = par_3/par_4 * 20000

